﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApplication1
{
    //Complete details and specifications:
    //http://www.n3fjp.com/help/api.html


    public partial class Form1 : Form
    {
        public EventDrivenTCPClient edc;   //Declare Event Driven Client

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            System.Net.IPAddress ip = System.Net.IPAddress.Parse(GetLocalIPAddress());  //Get IP Address of this PC.  N3FJP Software must also be running on this PC
            edc = new EventDrivenTCPClient(ip, 1100);  //Assign IP and port
            edc.DataReceived += new EventDrivenTCPClient.delDataReceived(client_DataReceived);  //Assign data received event
            edc.ConnectionStatusChanged += new EventDrivenTCPClient.delConnectionStatusChanged(client_ConnectionStatusChanged);  //Assign connection state change event
        }

        private void btcConnect_Click(object sender, EventArgs e)
        {
            edc.Connect(); //Connect to N3FJP Software
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            edc.Disconnect(); //Disconnect from N3FJP Software
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            edc.Send(txtSend.Text + "\r\n");  //Must include \r\n for the client to send the command!
        }
        
        private void client_ConnectionStatusChanged(EventDrivenTCPClient sender, EventDrivenTCPClient.ConnectionStatus status)
        {   //Fires on connection state change

            //Check if this event was fired on a different thread, if it is then we must invoke it on the UI thread
            if (InvokeRequired)
            {
                Invoke(new EventDrivenTCPClient.delConnectionStatusChanged(client_ConnectionStatusChanged), sender, status);
                return;
            }
            lblStatus.Text = "Connection: " + status.ToString();
        }

        private void client_DataReceived(EventDrivenTCPClient sender, object data)
        {   //Fires when data is received.  When actually implementing, use the buffer parsing code detailed here http://www.n3fjp.com/help/api.html
            //In case multiple commands arrive together

            //Again, check if this needs to be invoked in the UI thread
            if (InvokeRequired)
            {
                try
                {
                    Invoke(new EventDrivenTCPClient.delDataReceived(client_DataReceived), sender, data);
                }
                catch
                { }
                return;
            }
            //Interpret the received data object as a string
            string strData = data as string;
            //Add the received data to a text box
            txtRcvd.Text += strData + Environment.NewLine;  //Append new data to text box

            //Scroll to bottom of the text box
            txtRcvd.SelectionStart = txtRcvd.TextLength;
            txtRcvd.ScrollToCaret();
        }

        private string GetLocalIPAddress()  //Gets the local IP address for this PC
        {
            var host = System.Net.Dns.GetHostEntry(System.Net.Dns.GetHostName());
            foreach (var ip in host.AddressList)
            {
                if (ip.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
                {
                    this.Text = ip.ToString();
                    return ip.ToString();
                }
            }
            throw new Exception("Local IP Address Not Found!");
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {//Complete details and specifications:
            try
            {
                System.Diagnostics.Process.Start("http://www.n3fjp.com/help/api.html");
            }
            catch { }
        }
    }
}
